'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ExpressUnitError = undefined;
exports.Request = Request;
exports.Response = Response;
exports.run = run;

var _es6Error = require('es6-error');

var _es6Error2 = _interopRequireDefault(_es6Error);

var _express = require('express');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function Request() {
  this.app = {};
  this.body = {};
  this.query = {};
  this.route = {};
  this.params = {};
  this.headers = {};
  this.cookies = {};
  this.signedCookies = {};
}
Request.prototype = Object.create(_express.request);

function Response() {
  this.app = {}, this.locals = {};
}
Response.prototype = Object.create(_express.response);

const chainables = ['status', 'vary'];

function run(setup, middleware, done) {

  setup = setup || ((req, res, next) => next());

  const req = new Request();
  const res = new Response();

  let err = null;
  let isDone = false;

  const finish = (_err = null) => {
    err = _err;
    isFunction(done) && done(err, req, res);
    isDone = true;
  };

  for (let property in res) {
    if (isFunction(res[property])) {
      res[property] = () => chainables.includes(property) ? res : finish();
    }
  }

  let promise;

  setup(req, res, (_err = null) => {
    err = _err;
    promise = middleware.length <= 3 ? middleware(req, res, finish) : middleware(err, req, res, finish);
  });

  if (!isPromise(promise)) return;

  return promise.then(() => {
    if (isDone || !isFunction(done)) return [err, req, res];
    try {
      done(err, req, res);
    } catch (err) {
      throw new ExpressUnitError(null, err);
    }
  }).catch(err => {
    if (err instanceof ExpressUnitError) throw err.err;
    throw new ExpressUnitError('Unhandled rejection in middleware', err);
  });
}

class ExpressUnitError extends _es6Error2.default {
  constructor(message, err) {
    super(message);
    this.err = err;
  }
  toString() {
    const { name, message, err } = this;
    return `${ name }: ${ message }\n${ JSON.stringify(err, null, 2) }`;
  }
}

exports.ExpressUnitError = ExpressUnitError;
function isFunction(obj) {
  return typeof obj === 'function';
}

function isPromise(obj) {
  return obj && typeof obj === 'object' && isFunction(obj.then);
}